
%define ntpservers 131.225.8.127 131.225.17.127

# these macros are used below to help make future stuff
# easy to port from one thing to another
%define service ntpd
%define site fermi
%define firewallservice %{service}.%{site}

#
Summary: This will configure the ntpd config files for Fermilab. 
Name: zz_ntp_configure
Version: 4.2.6
Release: 6.slf
License: GPL
Group: Fermi
URL: http://fermilinux.fnal.gov/
Vendor: Fermilab
Packager: Pat Riehecky
Requires: ntp, coreutils, util-linux, perl, chkconfig, bash, augeas, /bin/awk 
Requires(post): ntp, coreutils, util-linux, perl, chkconfig, bash, augeas, /bin/awk
Source0: lsb_release
Source1: %{firewallservice}.slf5
Source2: %{firewallservice}.slf6
BuildArch: noarch

%description

This package configures the ntp time daemon correctly for use at Fermilab.  It
sets the initial time from the default time servers and configures ntpd to
listen to time broadcasts from the local router of the subdomain that the
machine is on.

At this time the default time servers are listed as:
%{ntpservers}

%prep
rm -rf $RPM_BUILD_ROOT

%install
mkdir -p $RPM_BUILD_ROOT/usr/share/doc/%{name}-%{version}/

%files
%defattr(755,root,root)
%doc $RPM_SOURCE_DIR/lsb_release
%doc $RPM_SOURCE_DIR/ntpd.fermi.slf5
%doc $RPM_SOURCE_DIR/ntpd.fermi.slf6

%post -p /bin/bash
# This copies in the correct firewall script for your version of SLF
VER=$(/usr/share/doc/%{name}-%{version}/lsb_release -r | awk '{ print $2}' | cut -d'.' -f1)
cp -f /usr/share/doc/%{name}-%{version}/%{firewallservice}.slf$VER /etc/sysconfig/%{firewallservice}

%preun
rm -f /etc/sysconfig/%{firewallservice}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -p /bin/bash -- ntp

UPGRADING_THE_RPM=0
if [[ $1 -ne 1 ]]; then
    # upgrading %{name}
    UPGRADING_THE_RPM=1
fi

UPGRADING_THE_TRIGGER_RPM=0
if [[ $2 -ne 1 ]]; then
    # upgrading what we trigger on
    UPGRADING_THE_TRIGGER_RPM=1
fi

if [[ ! -f /etc/sysconfig/%{firewallservice} ]]; then
    # This copies in the correct firewall script for your version of SLF
    # if somehow it was removed after %post, this will fix it back up...
    VER=$(/usr/share/doc/%{name}-%{version}/lsb_release -r | awk '{ print $2}' | cut -d'.' -f1)
    cp -f /usr/share/doc/%{name}-%{version}/%{firewallservice}.slf$VER /etc/sysconfig/%{firewallservice}
fi

#
# This modifies /etc/init.d/%{service} to poke holes in the firewall for ntp broadcasts
#
if [[ "$(grep /etc/sysconfig/%{firewallservice} /etc/init.d/%{service})" == "" ]] ; then
  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} start
  cat /etc/init.d/%{service} | sed 's:\(start().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} start\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  # adds /etc/sysconfig/%{firewallservice} to the execution of /etc/init.d/%{service} stop
  cat /etc/init.d/%{service} | sed 's:\(stop().*\):\1\nif \[ -x /etc/sysconfig/%{firewallservice} \] ; then \n/etc/sysconfig/%{firewallservice} stop\nfi:' > /etc/init.d/%{service}.tmp
  mv -f /etc/init.d/%{service}.tmp /etc/init.d/%{service}

  chmod 755 /etc/init.d/%{service}
fi

#
# This modifies /etc/ntp.conf
#
if [[ ! -f /etc/ntp.conf.rpmsave ]]; then   # make a backup before changes
    cp -fa /etc/ntp.conf /etc/ntp.conf.rpmsave
fi

# we want honesty rather than calculated expectations, don't fudge the numbers
grep -v '#' /etc/ntp.conf | grep 'fudge' 2>&1 >/dev/null
if [[ $? -eq 0 ]]; then
    TMPFILE=`mktemp`
    cat > $TMPFILE <<RMFUDGE
rm /files/etc/ntp.conf/fudge
save
RMFUDGE
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE
fi

# don't use TUV's ntp servers, we have our own
grep -v '#' /etc/ntp.conf | grep 'rhel.pool.ntp.org' 2>&1 >/dev/null
if [[ $? -eq 0 ]]; then
    TMPFILE=`mktemp`
    cat > $TMPFILE <<RMTUVNTPSERVERS
rm /files/etc/ntp.conf/server[ . =~ regexp(".*rhel.pool.ntp.org")]

save
RMTUVNTPSERVERS
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE
fi

# Speaking of our ntp servers, lets try broadcast shall we
TMPFILE=`mktemp`
cat > $TMPFILE <<SETBROADCASTCLIENT
set /files/etc/ntp.conf/broadcastclient

save
SETBROADCASTCLIENT
augtool < $TMPFILE >/dev/null 2>&1
rm -f $TMPFILE

# per RITM0088723
#   NTP clients shouldn't allow queries from 'just any' FNAL host
grep -v '#' /etc/ntp.conf | grep 'restrict 131.225.0.0' 2>&1 >/dev/null
if [[ $? -eq 0 ]]; then
    TMPFILE=`mktemp`
    cat > $TMPFILE <<RMFNALQ
rm /files/etc/ntp.conf/restrict[ . = '131.225.0.0' ]

save
RMFNALQ
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE
fi


# This makes sure the daemon is added to the runlevels 3, 4, and 5
# and restarts the service
/sbin/chkconfig --add ntpd
/sbin/chkconfig --levels 345 ntpd on
/etc/init.d/ntpd restart > /dev/null 2>&1

# give the ntp server a chance to look for broadcast peers
# takes about 20 seconds for broadcast peers to show up in peers list
sleep 20

# see how many lines of synced servers, there is always 1 line even
# when there are no synced servers, its a header

SYNCSERVERS=$(echo peers | ntpdc 2>/dev/null|wc -l) # simple querry for peers
if [[ $SYNCSERVERS -lt 4 ]]; then
    # be really really sure
    sleep 5  # give it a really good chance to catch up
    SYNCSERVERS=$(echo peers | ntpdc 2>/dev/null|wc -l)  # simple querry for peers
    if [[ $SYNCSERVERS -lt 4 ]]; then
        # if we aren't sync'd up to anything log it and add explicit servers
        # once added they will stay there unless something removes them
        logger -p daemon.notice -t %{name} "NTP Broadcast not detected, switching to manual servers."

        # The known good FNAL NTP servers should be added to the end
        for server in %{ntpservers}; do
             # asked not to use iburst
#            echo "server $server iburst" >> /etc/ntp.conf
            echo "server $server" >> /etc/ntp.conf
        done

        # make changes take effect
        /etc/init.d/ntpd restart > /dev/null 2>&1
    fi
fi

# during anaconda ntpd can't always be started
exit 0

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -- ntpdate
#
# This correctly configures /etc/ntp/step-tickers
#

if [ -s /etc/ntp/step-tickers ]; then
    if [ ! -f /etc/ntp/step-tickers.rpmsave ]; then
        cp -fa /etc/ntp/step-tickers /etc/ntp/step-tickers.rpmsave
    fi
fi
# insert the correct ntp servers into step-tickers
echo "%{ntpservers}" > /etc/ntp/step-tickers

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
%triggerin -- firstboot
#
# This makes it so that you don't get asked about ntp on the firstboot
# since it is already setup now.
# 
if [ -s /usr/share/firstboot/modules/date.py ]; then
	rm -f /usr/share/firstboot/modules/date.py*
fi

%changelog
* Tue Feb 18 2014 Pat Riehecky <riehecky@fnal.gov> 4.2.6-6
- removing "restrict 131.225.0.0 mask 255.255.0.0 nomodify noquery"
    per RITM0088723
- switched to augeas based config to try and simplify the transition

* Thu Jan 31 2013 Pat Riehecky <riehecky@fnal.gov> 4.2.6-5
- switched to 'bash' for triggers and post to avoid errors
- updated requires list to get things more in order

* Fri Mar 16 2012 Pat Riehecky <riehecky@fnal.gov> 4.2.6-4
- switched to a define based ntp server list
- updated ntp server list
- made version independent, now works on SLF5 and SLF6
- added comments to help explain the configuration options

* Thu Jun 16 2011 Connie Sieh <csieh@fnal.gov> 4.2.6-3
- Modified ntpd-fermi code from "RH-Firewall-1-INPUT" to "INPUT" as TUV
  Does not use "RH-Firewall-1-INPUT" anymore. 

* Thu May 26 2011 Tyler Parsons <tparsons@fnal.gov> 4.2.6-2
- added secondary and tertiary time servers to ntp.conf (131.225.17.150 and 131.225.8.120)
- changed references to servers from (131.225.8.200 and 131.225.17.200) to (131.225.17.150 and 131.225.8.120)
 
* Mon Aug 16 2010 Troy Dawson <dawson@fnal.gov> 4.2.6-1
- added ntpdate to the requirements
- Changed the step-tickers section to trigger on ntpdate
- Removed the change that added -A to /etc/sysconfig/ntpd

* Mon Jun 14 2010 Troy Dawson <dawson@fnal.gov> 4.2.0-9
- Added noquery to the restrict line

